#!/usr/bin/env python

"""
DOM wrapper around libxml2.
"""

__version__ = "0.2"

import xml.dom
import libxml2
from libxml2dom.macrolib import *
from libxml2dom.macrolib import \
    createDocument as Node_createDocument, \
    parseString as Node_parseString, parseURI as Node_parseURI, \
    parseFile as Node_parseFile, \
    toString as Node_toString, toStream as Node_toStream, \
    toFile as Node_toFile
import sys

# NOTE: Consider a generator instead.

class NamedNodeMap(object):

    def __init__(self, node):
        self.node = node

    def getNamedItem(self, name):
        return self.node.getAttributeNode(name)

    def getNamedItemNS(self, ns, localName):
        return self.node.getAttributeNodeNS(ns, localName)

    def setNamedItem(self, node):
        self.node.setAttributeNode(node.name, node)

    def setNamedItemNS(self, node):
        self.node.setAttributeNodeNS(node.namespaceURI, node.localName, node)

    def __getitem__(self, name):
        return self.getNamedItem(name)

    def __setitem__(self, name, node):
        if name == node.nodeName:
            self.setNamedItem(node)
        else:
            raise KeyError, name

    def __delitem__(self, name):
        # NOTE: To be implemented.
        pass

    def values(self):
        return [Node(_node) for _node in Node_attributes(self.node.as_native_node()).values()]

    def keys(self):
        return [(attr.namespaceURI, attr.localName) for attr in self.values()]

    def items(self):
        return [((attr.namespaceURI, attr.localName), attr) for attr in self.values()]

    def __repr__(self):
        return str(self)

    def __str__(self):
        return "{%s}" % ",\n".join(["%s : %s" % (repr(key), repr(value)) for key, value in self.items()])

class Node(object):

    def __init__(self, node, ownerElement=None, doctype=None):
        self._node = node

    def as_native_node(self):
        return self._node

    def _ownerDocument(self):
        return Node(Node_ownerDocument(self._node))

    def _nodeType(self):
        return Node_nodeType(self._node)

    def _childNodes(self):

        # NOTE: Consider a generator instead.

        return [Node(_node) for _node in Node_childNodes(self._node)]

    def _attributes(self):
        return NamedNodeMap(self)

    def _namespaceURI(self):
        return Node_namespaceURI(self._node)

    def _nodeValue(self):
        return Node_nodeValue(self._node)

    def _prefix(self):
        return Node_prefix(self._node)

    def _nodeName(self):
        return Node_nodeName(self._node)

    def _tagName(self):
        return Node_tagName(self._node)

    def _localName(self):
        return Node_localName(self._node)

    def _parentNode(self):
        return Node_parentNode(self._node)

    def _previousSibling(self):
        return Node(Node_previousSibling(self._node))

    def _nextSibling(self):
        return Node(Node_nextSibling(self._node))

    def hasAttributeNS(self, ns, localName):
        return Node_hasAttributeNS(self._node, ns, localName)

    def hasAttribute(self, name):
        return Node_hasAttribute(self._node, name)

    def getAttributeNS(self, ns, localName):
        return Node_getAttributeNS(self._node, ns, localName)

    def getAttribute(self, name):
        return Node_getAttribute(self._node, name)

    def getAttributeNodeNS(self, ns, localName):
        return Node(Node_getAttributeNodeNS(self._node, ns, localName))

    def getAttributeNode(self, localName):
        return Node(Node_getAttributeNode(self._node, localName))

    def setAttributeNS(self, ns, name, value):
        Node_setAttributeNS(self._node, ns, name, value)

    def setAttribute(self, name, value):
        Node_setAttribute(self._node, name, value)

    def setAttributeNodeNS(self, ns, name, node):
        Node_setAttributeNodeNS(self._node, ns, name, node)

    def setAttributeNode(self, name, node):
        Node_setAttributeNode(self._node, name, node)

    def createElementNS(self, ns, name):
        return Node(Node_createElementNS(self._node, ns, name))

    def createElement(self, name):
        return Node(Node_createElement(self._node, name))

    def createAttributeNS(self, ns, name):
        # Returns a special temporary node.
        return Node_createAttributeNS(self._node, ns, name)

    def createAttribute(self, name):
        # Returns a special temporary node.
        return Node_createAttribute(self._node, name)

    def createTextNode(self, value):
        return Node(Node_createTextNode(self._node, value))

    def createComment(self, value):
        return Node(Node_createComment(self._node, value))

    def importNode(self, node, deep):
        if hasattr(node, "as_native_node"):
            return Node(Node_importNode(self._node, node.as_native_node(), deep))
        else:
            return Node(Node_importNode_DOM(self._node, node, deep))

    def insertBefore(self, tmp, oldNode):
        if hasattr(tmp, "as_native_node"):
            return Node(Node_insertBefore(self._node, tmp.as_native_node(), oldNode.as_native_node()))
        else:
            return Node(Node_insertBefore(self._node, tmp, oldNode.as_native_node()))

    def replaceChild(self, tmp, oldNode):
        if hasattr(tmp, "as_native_node"):
            return Node(Node_replaceChild(self._node, tmp.as_native_node(), oldNode.as_native_node()))
        else:
            return Node(Node_replaceChild(self._node, tmp, oldNode.as_native_node()))

    def appendChild(self, tmp):
        if hasattr(tmp, "as_native_node"):
            return Node(Node_appendChild(self._node, tmp.as_native_node()))
        else:
            return Node(Node_appendChild(self._node, tmp))

    def removeChild(self, tmp):
        if hasattr(tmp, "as_native_node"):
            Node_removeChild(self._node, tmp.as_native_node())
        else:
            Node_removeChild(self._node, tmp)

    #doctype defined in __init__
    #ownerElement defined in __init__
    ownerDocument = property(_ownerDocument)
    childNodes = property(_childNodes)
    value = data = nodeValue = property(_nodeValue)
    name = nodeName = property(_nodeName)
    tagName = property(_tagName)
    namespaceURI = property(_namespaceURI)
    prefix = property(_prefix)
    localName = property(_localName)
    parentNode = property(_parentNode)
    nodeType = property(_nodeType)
    attributes = property(_attributes)
    previousSibling = property(_previousSibling)
    nextSibling = property(_nextSibling)

    #def isSameNode(self, other):
    #    return self._node.nodePath() == other._node.nodePath()

    #def __eq__(self, other):
    #    return self._node.nodePath() == other._node.nodePath()

    # 4DOM extensions to the usual PyXML API.
    # NOTE: To be finished.

    def xpath(self, expr, variables=None, namespaces=None):
        return [Node(_node) for _node in Node_xpath(self._node, expr, variables, namespaces)]

# Utility functions.

def createDocumentType(localName, publicId, systemId):
    return None

def createDocument(namespaceURI, localName, doctype):
    return Node(Node_createDocument(namespaceURI, localName, doctype))

def parse(stream_or_string):
    if hasattr(stream_or_string, "read"):
        stream = stream_or_string
        return parseString(stream.read())
    else:
        return parseFile(stream_or_string)

def parseFile(s):
    return Node(Node_parseFile(s))

def parseString(s):
    return Node(Node_parseString(s))

def parseURI(uri):
    return Node(Node_parseURI(uri))

def toString(node, encoding=None):
    return Node_toString(node.as_native_node(), encoding)

def toStream(node, stream, encoding=None):
    Node_toStream(node.as_native_node(), stream, encoding)

def toFile(node, f, encoding=None):
    Node_toFile(node.as_native_node(), f, encoding)

# vim: tabstop=4 expandtab shiftwidth=4
